/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (AboutDialog)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)GCAboutDialog.java    1.0 2001/06/07
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Insets;
import java.net.MalformedURLException;
import java.net.URL;
import javax.accessibility.AccessibleContext;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import org.mdanderson.biostat.util.DefaultAboutDialog;
import org.mdanderson.biostat.util.InetProtocolHandler;
import org.mdanderson.biostat.util.UrlLabel;


/**
 * GCAboutDialog - Creates an ABOUT dialog for this application.
 *
 * @version 1.0 2001/06/07
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class GCAboutDialog extends DefaultAboutDialog
    implements GeneClustKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying title
     *
     * @param title  string specifying dialog title
     */
    public GCAboutDialog(String title)
    {
        super(title);
    }

    /**
     * Class constructor specifying owner and title
     *
     * @param owner  parent component
     * @param title  string specifying dialog title
     */
    public GCAboutDialog(Frame owner, String title)
    {
        super(owner, title);
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Gets the preferred size of the component
     *
     * @return  dimension object indicating this component's preferred size
     */
    public Dimension getPreferredSize()
    {
        final int WIDTH = 420;
        final int HEIGHT = 270;

        return new Dimension(WIDTH, HEIGHT);
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Creates the banner panel displayed as the main pane
     * in the tabbed pane.
     *
     * @return <code>JPanel</code> representing about panel
     * @see GCAboutPanel
     */
    protected JPanel createAboutPanel()
    {
        return new GCAboutPanel();
    }

    /**
     * Creates a pane for displaying contact information
     *
     * @return <code>JPanel</code> representing contact panel
     */
    protected JPanel createContactPanel()
    {
        JPanel contactPanel = new JPanel();
        contactPanel.setForeground(Color.white);
        contactPanel.setLayout(new BoxLayout(contactPanel,
                                             BoxLayout.Y_AXIS));

        {
            TitledBorder titledBorder;
            String titleStr;

            titleStr = getBundleString(ABOUT_DIALOG_CONTACT_PANEL_TITLE);
            titledBorder = BorderFactory.createTitledBorder(titleStr);
            titledBorder.setTitleColor(Color.black);
            contactPanel.setBorder(titledBorder);
        }

        Font sansSerif = new Font("SansSerif", Font.PLAIN, 12);
        String propertyKey;
        String description;
        AccessibleContext ac;

        // Add email panel
        {
            propertyKey = ABOUT_DIALOG_CONTACT_EMAIL_ADDRESS_DESCRIPTION;
            description = getBundleString(propertyKey);
            propertyKey = ABOUT_DIALOG_CONTACT_EMAIL_ADDRESS;
            String emailAddress = getBundleString(propertyKey);
            URL url;

            try
            {
                final String protocol = "mailto";
                url = new URL(protocol + ":" + emailAddress);
            }
            catch (MalformedURLException mue)
            {
                url = null;
            }

            UrlLabel urlLabel = new UrlLabel(emailAddress, url);
            urlLabel.setFont(sansSerif);
            ac = urlLabel.getAccessibleContext();
            ac.setAccessibleDescription(description);
            urlLabel.addHyperlinkListener(inetProtocolHandler);

            propertyKey = ABOUT_DIALOG_CONTACT_EMAIL_LABEL;
            JLabel emailLabel = new JLabel(getBundleString(propertyKey));
            emailLabel.setForeground(Color.black);
            emailLabel.setFont(sansSerif);
            emailLabel.setLabelFor(urlLabel);

            JPanel emailPanel = new JPanel();
            emailPanel.add(emailLabel);
            emailPanel.add(urlLabel);

            contactPanel.add(emailPanel);
        }

        // Add web panel
        {
            propertyKey = ABOUT_DIALOG_CONTACT_WEB_ADDRESS_DESCRIPTION;
            description = getBundleString(propertyKey);
            propertyKey = ABOUT_DIALOG_CONTACT_WEB_ADDRESS;
            String webAddress = getBundleString(propertyKey);
            URL url;

            try
            {
                final String protocol = "http";
                url = new URL(protocol + "://" + webAddress);
            }
            catch (MalformedURLException mue)
            {
                url = null;
            }

            UrlLabel urlLabel = new UrlLabel(webAddress, url);
            urlLabel.setFont(sansSerif);
            ac = urlLabel.getAccessibleContext();
            ac.setAccessibleDescription(description);
            urlLabel.addHyperlinkListener(inetProtocolHandler);

            propertyKey = ABOUT_DIALOG_CONTACT_WEB_LABEL;
            JLabel webLabel = new JLabel(getBundleString(propertyKey));
            webLabel.setForeground(Color.black);
            webLabel.setFont(sansSerif);
            webLabel.setLabelFor(urlLabel);

            JPanel webPanel = new JPanel();
            webPanel.add(webLabel);
            webPanel.add(urlLabel);

            contactPanel.add(webPanel);
        }

        // Add usps panel
        {
            propertyKey = ABOUT_DIALOG_CONTACT_USPS_ADDRESS_NAME;
            JLabel nameLabel = new JLabel(getBundleString(propertyKey));
            nameLabel.setFont(sansSerif);
            nameLabel.setForeground(Color.black);

            propertyKey = ABOUT_DIALOG_CONTACT_USPS_ADDRESS_STREET;
            JLabel addrLabel = new JLabel(getBundleString(propertyKey));
            addrLabel.setFont(sansSerif);
            addrLabel.setForeground(Color.black);

            propertyKey = ABOUT_DIALOG_CONTACT_USPS_ADDRESS_CITY_STATE;
            JLabel cityLabel = new JLabel(getBundleString(propertyKey));
            cityLabel.setFont(sansSerif);
            cityLabel.setForeground(Color.black);

            JPanel uspsInfoPanel = new JPanel();
            uspsInfoPanel.setLayout(new BoxLayout(uspsInfoPanel,
                                                  BoxLayout.Y_AXIS));
            uspsInfoPanel.add(nameLabel);
            uspsInfoPanel.add(addrLabel);
            uspsInfoPanel.add(cityLabel);

            propertyKey = ABOUT_DIALOG_CONTACT_USPS_LABEL;
            JLabel uspsLabel = new JLabel(getBundleString(propertyKey));
            uspsLabel.setForeground(Color.black);
            uspsLabel.setFont(sansSerif);
            uspsLabel.setLabelFor(uspsInfoPanel);

            JPanel uspsPanel = new JPanel();
            uspsPanel.add(uspsLabel);
            uspsPanel.add(uspsInfoPanel);

            contactPanel.add(uspsPanel);
        }

        return contactPanel;
    }

    /**
     * Creates a pane for displaying developer credits, et cetera.
     *
     * @return <code>JPanel</code> representing credits panel
     */
    protected JPanel createCreditsPanel()
    {
        final Dimension hdim;
        final Dimension vdim;

        {
            final int HGAP = 45;
            final int VGAP = 15;

            hdim = new Dimension(HGAP, 0);
            vdim = new Dimension(0, VGAP);
        }

        JPanel creditsPanel = new JPanel();

        creditsPanel.setLayout(new BoxLayout(creditsPanel, BoxLayout.Y_AXIS));
        creditsPanel.add(Box.createRigidArea(hdim));

        // Add elements
        {
            JLabel label;
            String propertyKey;
            Font headingFont = new Font("SansSerif", Font.BOLD, 12);
            Font entryFont = new Font("Monospaced", Font.PLAIN, 12);

            // Add researcher labels
            {
                JLabel researcherLabel;

                researcherLabel = new JLabel(kim);
                researcherLabel.setForeground(Color.black);
                researcherLabel.setFont(entryFont);
                researcherLabel.setLabelFor(null);

                propertyKey = ABOUT_DIALOG_CREDITS_RESEARCHER_LABEL;
                label = new JLabel(getBundleString(propertyKey));
                label.setForeground(Color.black);
                label.setFont(headingFont);
                label.setLabelFor(researcherLabel);

                creditsPanel.add(Box.createRigidArea(vdim));
                creditsPanel.add(label);
                creditsPanel.add(researcherLabel);
                creditsPanel.add(Box.createRigidArea(vdim));
            }

            // Add developer labels
            {
                JLabel developerLabel;
                JPanel developerPanel = new JPanel();
                developerPanel.setLayout(new BoxLayout(developerPanel,
                                                       BoxLayout.Y_AXIS));

                developerLabel = new JLabel(rumiana);
                developerLabel.setForeground(Color.black);
                developerLabel.setFont(entryFont);
                developerLabel.setLabelFor(null);
                developerPanel.add(developerLabel);

                developerLabel = new JLabel(paul);
                developerLabel.setForeground(Color.black);
                developerLabel.setFont(entryFont);
                developerLabel.setLabelFor(null);
                developerPanel.add(developerLabel);

                propertyKey = ABOUT_DIALOG_CREDITS_DEVELOPER_LABEL;
                label = new JLabel(getBundleString(propertyKey));
                label.setForeground(Color.black);
                label.setFont(headingFont);
                label.setLabelFor(developerPanel);

                creditsPanel.add(label);
                creditsPanel.add(developerPanel);
                creditsPanel.add(Box.createRigidArea(vdim));
            }

            // Add collaborator labels
            {
                JLabel collaboratorLabel;
                JPanel collaboratorPanel = new JPanel();
                collaboratorPanel.setLayout(new BoxLayout(collaboratorPanel,
                                                          BoxLayout.Y_AXIS));

                collaboratorLabel = new JLabel(bradley);
                collaboratorLabel.setForeground(Color.black);
                collaboratorLabel.setFont(entryFont);
                collaboratorLabel.setLabelFor(null);
                collaboratorPanel.add(collaboratorLabel);

                collaboratorLabel = new JLabel(keith);
                collaboratorLabel.setForeground(Color.black);
                collaboratorLabel.setFont(entryFont);
                collaboratorLabel.setLabelFor(null);
                collaboratorPanel.add(collaboratorLabel);

                propertyKey = ABOUT_DIALOG_CREDITS_COLLABORATOR_LABEL;
                label = new JLabel(getBundleString(propertyKey));
                label.setForeground(Color.black);
                label.setFont(headingFont);
                label.setLabelFor(collaboratorPanel);

                creditsPanel.add(label);
                creditsPanel.add(collaboratorPanel);
                creditsPanel.add(Box.createRigidArea(vdim));
            }
        }

        return creditsPanel;
    }

    // =========================================================================
    // MAIN
    // =========================================================================

    /**
     * Main entry-point for the GCAboutDialog class
     *
     * @param argv  string array containing command line arguments
     */
    public static void main(String[] argv)
    {
        final String title = getBundleString(ABOUT_DIALOG_TITLE);
        GCAboutDialog about = new GCAboutDialog(title) {
            public void dispose()
            {
                super.dispose();
                System.exit(0);
            }
        };

        // Set preferred dimensions
        {
            final int WIDTH  = (int) about.getPreferredSize().getWidth();
            final int HEIGHT = (int) about.getPreferredSize().getHeight();

            Insets insets = about.getInsets();
            int width  = WIDTH  + (insets.left + insets.right);
            int height = HEIGHT + (insets.top + insets.bottom);
System.out.println("WIDTH: " + WIDTH + "  HEIGHT: " + HEIGHT);
System.out.println("width: " + width + "  height: " + height);
            about.setSize(width, height);
        }

        about.display();
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return GeneClust.getBundleString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static String bradley;
    private static String keith;
    private static String kim;
    private static String paul;
    private static String rumiana;
    static InetProtocolHandler inetProtocolHandler;

    static
    {
        bradley = "\u0042\u0072\u0061\u0064\u006c\u0065\u0079" + " " +
                  "\u0042\u0072\u006f\u006f\u006d";
        keith   = "\u004b\u0065\u0069\u0074\u0068" + " " +
                  "\u0042\u0061\u0067\u0067\u0065\u0072\u006c\u0079";
        kim     = "\u004b\u0069\u006d\u002d\u0041\u006e\u0068" + " " +
                  "\u0044\u006f";
        paul    = "\u0050\u0061\u0075\u006c" + " " +
                  "\u0052\u006f\u0065\u0062\u0075\u0063\u006b";
        rumiana = "\u0052\u0075\u006d\u0069\u0061\u006e\u0061" + " " +
                  "\u004e\u0069\u006b\u006f\u006c\u006f\u0076\u0061";

        inetProtocolHandler = new InetProtocolHandler();
    }
}

