/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (Settings)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)Settings.java    0.95 2001/02/13
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;


/**
 * Settings - Load and store settings for user interface values.
 *
 * @version 0.99 2001/05/18
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class Settings
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor
     */
    public Settings()
    {
    }

    /**
     * Class constructor specifying if demo enabled
     *
     * @param isDemoEnabled  if true, set demo enabled; otherwise, disable it
     */
    public Settings(boolean isDemoEnabled)
    {
        setSelectedMode(isDemoEnabled);
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Reads the settings file and updates the associated GUI components
     * with those values.
     *
     * @param file  abstract pathname containing properties to read
     * @throws FileNotFoundException         if file could not be located
     * @throws FileVersionMismatchException  if file created by more recent
     *                                       version of this software
     */
    public void read(File file)
      throws FileVersionMismatchException, FileNotFoundException
    {
        try
        {
            Properties p = new Properties();
            String fileVersion;

            FileInputStream fis = new FileInputStream(file);
            p.load(fis);

            fileVersion = p.getProperty(keyFileVersion);
            if (fileVersion.compareTo(currentVersion) > 0)
                throw new FileVersionMismatchException(fileVersion);

            selectedMode = p.getProperty(keySelectedMode);

            getGenericSettings(p);
            getRawDataSettings(p);
            getDemoSettings(p);
        }
        catch (FileNotFoundException fnfe)
        {
            throw fnfe;
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }

    /**
     * Writes the current GUI component values to a settings file.
     *
     * @param file  abstract pathname where properties are to be saved
     */
    public void write(File file)
    {
        try
        {
            Properties p = new Properties();

            p.setProperty(keyFileVersion, currentVersion);
            p.setProperty(keySelectedMode, selectedMode);

            setGenericSettings(p);
            setRawDataSettings(p);
            setDemoSettings(p);

            FileOutputStream fos = new FileOutputStream(file);
            p.store(fos, identMessage);
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }

    // =========================================================================
    // PUBLIC INTERFACE (Accessors)
    // =========================================================================

    /**
     * Returns the selected simulation mode.
     *
     * @return  true if demo mode selected; otherwise, false (raw data mode)
     * @see #setSelectedMode
     */
    public boolean isDemoMode()
    {
        return (this.selectedMode.compareTo(DEMO_MODE) == 0);
    }

    /**
     * Returns the selected simulation mode.
     *
     * @return  string representing the selected mode
     * @see #setSelectedMode
     */
    public String getSelectedMode()
    {
        return selectedMode;
    }

    /**
     * Returns the flag used to determine if genes graph should be displayed.
     *
     * @return   true if genes graph should be displayed
     * @see #setIsDisplayGenesGraph
     */
    public boolean getIsDisplayGenesGraph()
    {
        return isDisplayGenesGraph;
    }

    /**
     * Returns the flag used to determine if samples graph should be displayed.
     *
     * @return   true if samples graph should be displayed
     * @see #setIsDisplaySamplesGraph
     */
    public boolean getIsDisplaySamplesGraph()
    {
        return isDisplaySamplesGraph;
    }

    /**
     * Returns the flag used to determine if samples graph should be displayed.
     *
     * @return   true if samples graph should be displayed
     * @see #setIsDisplaySamplesGraph
     */
    public boolean getIsDisplayDataMatrix()
    {
        return isDisplayDataMatrix;
    }

    /**
     * Returns the flag used to determine if samples graph should be displayed.
     *
     * @return   true if samples graph should be displayed
     * @see #setIsDisplaySamplesGraph
     */
    public boolean getIsDisplayShaving()
    {
        return isDisplayShaving;
    }

    /**
     * Returns the flag used to determine if supervised shaving should be
     * performed.
     *
     * @return   true if supervised shaving enabled; otherwise,
     *           false if supervised shaving disabled
     * @see #setIsPerformSupervised
     */
    public boolean getIsPerformSupervised()
    {
        return isPerformSupervised;
    }

    /**
     * Returns the supervision percentage.
     *
     * @return  string containing percent supervision
     * @see #setPercentSupervision
     */
    public String getPercentSupervision()
    {
        return percentSupervision;
    }

    /**
     * Returns the classification file to use for supervised shaving.
     *
     * @return  string containing classification file name
     * @see #setClassificationFileName
     */
    public String getClassificationFileName()
    {
        return classificationFileName;
    }

    /**
     * Returns the number of clusters to extract.
     *
     * @return  string containing number of clusters to extract
     * @see #setNumExtClusters
     */
    public String getNumExtClusters()
    {
        return numExtClusters;
    }

    /**
     * Returns the percent to shave.
     *
     * @return  string containing percent to shave
     * @see #setPercentToShave
     */
    public String getPercentToShave()
    {
        return percentToShave;
    }

    /**
     * Returns the number of permutations to find optimal cluster.
     *
     * @return  string containing number of permutations
     * @see #setNumPermutations
     */
    public String getNumPermutations()
    {
        return numPermutations;
    }

    /**
     * Returns the gap tolerance.
     *
     * @return  string containing gap tolerance
     * @see #setGapTolerance
     */
    public String getGapTolerance()
    {
        return gapTolerance;
    }

    /**
     * Returns the raw data file to process.
     *
     * @return  string containing raw data file name
     * @see #setRawDataFileName
     */
    public String getRawDataFileName()
    {
        return rawDataFileName;
    }

    /**
     * Returns the number of rows in the gene expression matrix (genes)
     *
     * @return  string containing number of rows
     * @see #setNumRows
     */
    public String getNumRows()
    {
        return numRows;
    }

    /**
     * Returns the number of samples in the gene expression matrix (cell lines)
     *
     * @return  string containing number of columns
     * @see #setNumColumns
     */
    public String getNumColumns()
    {
        return numColumns;
    }

    /**
     * Returns the maximum value of signal used when generating matrix
     *
     * @return  string containing amplitude
     * @see #setAmplitude
     */
    public String getAmplitude()
    {
        return amplitude;
    }

    /**
     * Returns the model to use when generating matrix
     *
     * @return  string containing model (data generation type)
     * @see #setModel
     */
    public String getModel()
    {
        return model;
    }

    /**
     * Returns the type of noise to use when generating matrix
     *
     * @return  string containing noise to generate
     *          (none, Gaussian, Chi-square, or T)
     * @see #setNoise
     */
    public String getNoise()
    {
        return noise;
    }

    /**
     * Returns the degrees of freedom needed by certain algorithms
     * for noise generation
     *
     * @return  string containing degrees of freedom
     * @see #setDegreesFreedom
     * @see #setNoise
     */
    public String getDegreesFreedom()
    {
        return degreesFreedom;
    }

    // =========================================================================
    // PUBLIC INTERFACE (Mutators)
    // =========================================================================

    /**
     * Sets the selected simulation mode.
     *
     * @param isDemoEnabled  if true, set demo mode; otherwise, raw data mode
     * @see #setSelectedMode(String)
     * @see #getSelectedMode
     * @see #isDemoMode
     */
    public void setSelectedMode(boolean isDemoEnabled)
    {
        this.selectedMode = (isDemoEnabled) ? DEMO_MODE : RAW_DATA_MODE;
    }

    /**
     * Sets the selected simulation mode.
     *
     * @param mode  string representing the selected mode
     * @see #setSelectedMode(boolean)
     * @see #getSelectedMode
     * @see #isDemoMode
     */
    public void setSelectedMode(String mode)
    {
        this.selectedMode = mode;
    }

    /**
     * Sets the flag used to determine if genes graph should be displayed.
     *
     * @param isDisplayGenesGraph  if true, enable genes graph display
     * @see #getIsDisplayGenesGraph
     */
    public void setIsDisplayGenesGraph(boolean isDisplayGenesGraph)
    {
        this.isDisplayGenesGraph = isDisplayGenesGraph;
    }

    /**
     * Sets the flag used to determine if samples graph should be displayed.
     *
     * @param isDisplaySamplesGraph  if true, enable samples graph display
     * @see #getIsDisplaySamplesGraph
     */
    public void setIsDisplaySamplesGraph(boolean isDisplaySamplesGraph)
    {
        this.isDisplaySamplesGraph = isDisplaySamplesGraph;
    }

    /**
     * Sets the flag used to determine if samples graph should be displayed.
     *
     * @param isDisplayDataMatrix  if true, enable samples graph display
     * @see #getIsDisplayDataMatrix
     */
    public void setIsDisplayDataMatrix(boolean isDisplayDataMatrix)
    {
        this.isDisplayDataMatrix = isDisplayDataMatrix;
    }

    /**
     * Sets the flag used to determine if samples graph should be displayed.
     *
     * @param isDisplayShaving  if true, enable samples graph display
     * @see #getIsDisplayShaving
     */
    public void setIsDisplayShaving(boolean isDisplayShaving)
    {
        this.isDisplayShaving = isDisplayShaving;
    }

    /**
     * Sets the flag used to determine if supervised shaving should be performed
     *
     * @param isPerformSupervised  if true, enable supervised shaving
     * @see #getIsPerformSupervised
     */
    public void setIsPerformSupervised(boolean isPerformSupervised)
    {
        this.isPerformSupervised = isPerformSupervised;
    }

    /**
     * Sets the supervision percentage.
     *
     * @param percentSupervision  string containing percent to supervise
     * @see #getPercentSupervision
     */
    public void setPercentSupervision(final String percentSupervision)
    {
        this.percentSupervision = percentSupervision;
    }

    /**
     * Sets the classification file to use for supervised shaving.
     *
     * @param classificationFileName  string containing classification file name
     * @see #getClassificationFileName
     */
    public void setClassificationFileName(String classificationFileName)
    {
        this.classificationFileName = classificationFileName;
    }

    /**
     * Sets the number of clusters to extract.
     *
     * @param numExtClusters  string containing number of clusters to extract
     * @see #getNumExtClusters
     */
    public void setNumExtClusters(final String numExtClusters)
    {
        this.numExtClusters = numExtClusters;
    }

    /**
     * Sets the percent to shave.
     *
     * @param percentToShave  string containing percent to shave
     * @see #getPercentToShave
     */
    public void setPercentToShave(final String percentToShave)
    {
        this.percentToShave = percentToShave;
    }

    /**
     * Sets the number of permutations to find optimal cluster.
     *
     * @param numPermutations  string containing number of permutations
     * @see #getNumPermutations
     */
    public void setNumPermutations(final String numPermutations)
    {
        this.numPermutations = numPermutations;
    }

    /**
     * Sets the gap tolerance.
     *
     * @param gapTolerance  string containing gap tolerance
     * @see #getGapTolerance
     */
    public void setGapTolerance(final String gapTolerance)
    {
        this.gapTolerance = gapTolerance;
    }

    /**
     * Sets the raw data file to process.
     *
     * @param rawDataFileName  string containing raw data file name
     * @see #getRawDataFileName
     */
    public void setRawDataFileName(String rawDataFileName)
    {
        this.rawDataFileName = rawDataFileName;
    }

    /**
     * Sets the number of rows in the gene expression matrix (genes)
     *
     * @param numRows  string containing number of rows
     * @see #getNumRows
     */
    public void setNumRows(String numRows)
    {
        this.numRows = numRows;
    }

    /**
     * Sets the number of samples in the gene expression matrix (cell lines)
     *
     * @param numColumns  string containing number of columns
     * @see #getNumColumns
     */
    public void setNumColumns(String numColumns)
    {
        this.numColumns = numColumns;
    }

    /**
     * Sets the maximum value of signal used when generating matrix
     *
     * @param amplitude  string containing amplitude
     * @see #getAmplitude
     */
    public void setAmplitude(String amplitude)
    {
        this.amplitude = amplitude;
    }

    /**
     * Sets the model to use when generating matrix
     *
     * @param model  string containing model (data generation type)
     * @see #getModel
     */
    public void setModel(String model)
    {
        this.model = model;
    }

    /**
     * Sets the type of noise to use when generating matrix
     *
     * @param noise  string containing noise to generate
     *               (none, Gaussian, Chi-square, or T)
     * @see #getNoise
     */
    public void setNoise(String noise)
    {
        this.noise = noise;
    }

    /**
     * Sets the degrees of freedom nneded by certain algorithms
     * for noise generation
     *
     * @param degreesFreedom  string containing degrees of freedom
     * @see #getDegreesFreedom
     * @see #setNoise
     */
    public void setDegreesFreedom(String degreesFreedom)
    {
        this.degreesFreedom = degreesFreedom;
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Copies mode-indifferent property values from <code>Properties</code>
     * object to instance variables.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void getGenericSettings(Properties p)
    {
        String displayGenesGraph;
        String displaySamplesGraph;
        String displayDataMatrix;
        String displayShaving;
        String performSupervised;

        numExtClusters         = p.getProperty(keyNumExtClusters);
        percentToShave         = p.getProperty(keyPercentToShave);
        numPermutations        = p.getProperty(keyNumPermutations);
        gapTolerance           = p.getProperty(keyGapTolerance);
        displayGenesGraph      = p.getProperty(keyDisplayGenesGraph);
        displaySamplesGraph    = p.getProperty(keyDisplaySamplesGraph);
        displayDataMatrix    = p.getProperty(keyDisplayDataMatrix);
        displayShaving    = p.getProperty(keyDisplayShaving);
        performSupervised      = p.getProperty(keyPerformSupervised);
        percentSupervision     = p.getProperty(keyPercentSupervision);
        classificationFileName = p.getProperty(keyClassificationFileName);

        isDisplayGenesGraph   = new Boolean(displayGenesGraph).booleanValue();
        isDisplaySamplesGraph = new Boolean(displaySamplesGraph).booleanValue();
        isDisplayDataMatrix = new Boolean(displayDataMatrix).booleanValue();
        isDisplayShaving = new Boolean(displayShaving).booleanValue();
        isPerformSupervised   = new Boolean(performSupervised).booleanValue();
    }

    /**
     * Copies mode-indifferent property values from instance variables to
     * <code>Properties</code> object.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void setGenericSettings(Properties p)
    {
        p.setProperty(keyNumExtClusters,  numExtClusters);
        p.setProperty(keyPercentToShave,  percentToShave);
        p.setProperty(keyNumPermutations, numPermutations);
        p.setProperty(keyGapTolerance,    gapTolerance);
        p.setProperty(keyDisplayGenesGraph,
                      new Boolean(isDisplayGenesGraph).toString());
        p.setProperty(keyDisplaySamplesGraph,
                      new Boolean(isDisplaySamplesGraph).toString());
        p.setProperty(keyDisplayDataMatrix,
                      new Boolean(isDisplayDataMatrix).toString());
        p.setProperty(keyDisplayShaving,
                      new Boolean(isDisplayShaving).toString());
        p.setProperty(keyPerformSupervised,
                      new Boolean(isPerformSupervised).toString());
        p.setProperty(keyPercentSupervision,  percentSupervision);
        p.setProperty(keyClassificationFileName, classificationFileName);
    }

    /**
     * Copies raw data mode property values from <code>Properties</code>
     * object to instance variables.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void getRawDataSettings(Properties p)
    {
        rawDataFileName = p.getProperty(keyRawDataFileName);
    }

    /**
     * Copies raw data mode property values from instance variables to
     * <code>Properties</code> object.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void setRawDataSettings(Properties p)
    {
        p.setProperty(keyRawDataFileName, rawDataFileName);
    }

    /**
     * Copies demo mode property values from <code>Properties</code>
     * object to instance variables.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void getDemoSettings(Properties p)
    {
        numRows        = p.getProperty(keyNumRows);
        numColumns     = p.getProperty(keyNumColumns);
        amplitude      = p.getProperty(keyAmplitude);
        model          = p.getProperty(keyModel);
        noise          = p.getProperty(keyNoise);
        degreesFreedom = p.getProperty(keyDegreesFreedom);
    }

    /**
     * Copies demo mode property values from instance variables to
     * <code>Properties</code> object.
     *
     * @param p  reference to <code>Properties</code> object
     */
    private void setDemoSettings(Properties p)
    {
        p.setProperty(keyNumRows,        numRows);
        p.setProperty(keyNumColumns,     numColumns);
        p.setProperty(keyAmplitude,      amplitude);
        p.setProperty(keyModel,          model);
        p.setProperty(keyNoise,          noise);
        p.setProperty(keyDegreesFreedom, degreesFreedom);
    }

    // =========================================================================
    // CONSTANTS
    // =========================================================================

    private static final String RAW_DATA_MODE = "raw data";
    private static final String DEMO_MODE     = "demo";

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String identMessage = "GeneClust Configuration File";
    private static final String currentVersion = "1";

    private static final String keyFileVersion     = "version";
    private static final String keySelectedMode    = "selected_mode";

    private static final String keyDisplayGenesGraph = "display_genes_graph";
    private static final String keyDisplaySamplesGraph = "display_samples_graph";
    private static final String keyDisplayDataMatrix = "display_data_matrix";
    private static final String keyDisplayShaving = "display_shaving";
    private static final String keyPerformSupervised = "perform_supervised";
    private static final String keyPercentSupervision = "percent_supervision";
    private static final String keyClassificationFileName =
                                                     "classification_file_name";
    private static final String keyNumExtClusters  = "num_clusters_to_extract";
    private static final String keyPercentToShave  = "percent_to_shave";
    private static final String keyNumPermutations = "num_permutations";
    private static final String keyGapTolerance    = "gap_tolerance";
    private static final String keyRawDataFileName = "raw_data_file_name";
    private static final String keyNumRows         = "num_rows";
    private static final String keyNumColumns      = "num_columns";
    private static final String keyAmplitude       = "amplitude";
    private static final String keyModel           = "model";
    private static final String keyNoise           = "noise";
    private static final String keyDegreesFreedom  = "degrees_of_freedom";

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private String selectedMode;

    // Generic settings
    private String numExtClusters;
    private String percentToShave;
    private String numPermutations;
    private String gapTolerance;
    private boolean isDisplayGenesGraph;
    private boolean isDisplaySamplesGraph;
    private boolean isDisplayDataMatrix;
    private boolean isDisplayShaving;
    private boolean isPerformSupervised;
    private String percentSupervision;
    private String classificationFileName;

    // Raw Data settings
    private String rawDataFileName;

    // Demo settings
    private String numRows;
    private String numColumns;
    private String amplitude;
    private String model;
    private String noise;
    private String degreesFreedom;
}


/**
 * FileVersionMismatchException - Thrown when file version mismatch occurs.
 *
 * @version 0.95 2001/02/13
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class FileVersionMismatchException extends Exception
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying message
     *
     * @param message  string containing detail message
     */
    public FileVersionMismatchException(String message)
    {
        super(message);
    }
}

