/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (Simulation)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)Simulation.java    0.95 2001/01/29
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.io.BufferedWriter;
import java.io.IOException;

abstract public class Simulation extends SProcess
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying (way too many) simulation parameters...
     *
     * @param dialog                  dialog allows monitoring of sim progress
     * @param isPerformSupervised     if true, perform supervised shaving
     * @param percentSupervision      percent supervision to use
     * @param classificationPathName  classification for supervised shaving
     * @param numClustersToExtract    number of clusters to extract
     * @param percentToShave          percent to shave
     * @param numPermutations         number of permutations
     * @param gapTolerance            gap tolerance
     * @param isWriteMatrix           if true, write matrix to disk. This is
     *                                only useful in demo mode.
     * @throws NullPointerException if dialog is null
     */
    public Simulation(SimulationDialog dialog,
                      boolean isPerformSupervised,
                      int percentSupervision,
                      String classificationPathName,
                      int numClustersToExtract,
                      int percentToShave,
                      int numPermutations,
                      int gapTolerance,
                      boolean isWriteMatrix)
    {
	super(dialog);

        Boolean isSupervised = new Boolean(isPerformSupervised);
        Boolean isWrite = new Boolean(isWriteMatrix);

        this.performSupervised = isSupervised.toString().toUpperCase();
        this.percentSupervision = percentSupervision;
        this.classificationFileName = classificationPathName;
        this.numClustersToExtract = numClustersToExtract;
        this.percentToShave = percentToShave;
        this.numPermutations = numPermutations;
        this.gapTolerance = gapTolerance;
        this.writeMatrix = isWrite.toString().toUpperCase();
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Writes the S interpreter initialization code. It sets S interpreter options and global
     * variables, dynamically loads the C library, and initializes the S interpreter
     * graphics environment.
     *
     * @param bw  output stream writer
     * @throws IOException if an error occurs while writing
     */
    protected void writeSinterpreterInit(SBufferedWriter bw)
      throws IOException
    {
        //bw.Command("options(echo=TRUE, interactive=TRUE)");
        bw.Command ("options(echo=TRUE);");
	bw.SetStringVar ("GCinstall",     installDirName);
	bw.SetStringVar ("GCinterpreter", interpreterName);
	bw.SetStringVar ("GCoutput",      outputDirName);

	bw.Source ("GCObjectLoader.s");
	bw.Source ("DisplayCluster.s");
	bw.Source ("GeneClust.s");
        bw.Command("options(error=ErrorHandler.func);");

	bw.Source ("Graphics.s");
        bw.Command("InitGraphics.func();");

        bw.write("cat(\"PK_MONITOR_APP_PROCESSING_INPUT\\n\")");
        bw.newLine();

        bw.flush();
    }

    /**
     * Writes mode-indifferent S interpreter input data to the output stream.
     *
     * @param bw  output stream writer
     * @throws IOException if an error occurs while writing
     */
    protected void writeSinterpreterGenericInput(SBufferedWriter bw)
      throws IOException
    {
        bw.newLine();
        if (new Boolean(performSupervised).booleanValue() == true)
        {
            bw.write("percentSupervision<-" + percentSupervision);
            bw.newLine();
            bw.SetStringVar("classfile", classificationFileName);
        }
        else
        {
            bw.write("percentSupervision<-0");
            bw.newLine();
            bw.write("classfile<-NULL");
            bw.newLine();
        }
        bw.SetIntegerVar("nclst", numClustersToExtract);
        bw.write("percentToShave<-" + percentToShave);
        bw.newLine();
        bw.SetIntegerVar ("nperm", numPermutations);
        bw.write("gapTolerance<-" + gapTolerance);
        bw.newLine();
        bw.SetStringVar ("writeMatrix", writeMatrix); // Actually a Logical...
    }

    // =========================================================================
    // DEFAULT INTERFACE
    // =========================================================================

    /**
     * Returns a string consisting of <i>length</i> number of <i>padChar</i>.
     *
     * @param length   number of pad characters
     * @param padChar  character to use
     * @return         string of specified length consisting of specified char
     */
    String padding(int length, char padChar)
    {
        char[] buffer = new char[length];
        for (int i = 0; i < length; i++)
            buffer[i] = padChar;

        return new String(buffer);
    }

    /**
     * Returns a string containing the specified number of spaces. Used to make
     * S interpreter input "line up" when visually inspected.
     *
     * @param length  number of pad characters
     * @return        blank string
     */
    String blankPadding(int length)
    {
        return padding(length, ' ');
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static String installDirName;
    private static String interpreterName;
    private static String outputDirName;

    static
    {
        final String keyGCInstallDir = "gcinstall.home";
        final String keyGCOutputDir = "geneclust.output";
        final String keyGCInterpreterName = "sinterpreter.name";

        installDirName = System.getProperty(keyGCInstallDir);
        interpreterName = System.getProperty(keyGCInterpreterName);
        outputDirName = System.getProperty(keyGCOutputDir);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected String performSupervised;
    protected int percentSupervision;
    protected String classificationFileName;
    protected int numClustersToExtract;
    protected int percentToShave;
    protected int numPermutations;
    protected int gapTolerance;
    protected String writeMatrix;
}


/**
 * DemoSimulation - Data passing between the user interface and Sinterpreter when
 *                  running in demo mode.
 *
 * @version 0.95 2001/02/22
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
final class DemoSimulation extends Simulation
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor specifying (way too many) simulation parameters...
     *
     * @param dialog                  dialog allows monitoring of sim progress
     * @param isPerformSupervised     if true, perform supervised shaving
     * @param percentSupervision      percent supervision to use
     * @param classificationPathName  classification for supervised shaving
     * @param numClustersToExtract    number of clusters to extract
     * @param percentToShave          percent to shave
     * @param numPermutations         number of permutations
     * @param gapTolerance            gap tolerance
     * @param isWriteMatrix           if true, write matrix to disk; otherwise,
     *                                generated data cannot be viewed if needed.
     * @param numRows                 number of rows in gene expression matrix
     *                                (genes)
     * @param numColumns              number of samples (cell lines)
     * @param amplitude               amplitude used in data generation
     * @param model                   model specifies type of data generation 
     * @param noise                   noise to generate
     *                                (none, Gaussian, Chi-square, or T)
     * @param degreesFreedom          degrees of freedom needed by certain
     *                                algorithms for noise generation
     * @throws NullPointerException if dialog is null
     */
    public DemoSimulation(SimulationDialog dialog,
                          boolean isPerformSupervised,
                          int percentSupervision,
                          String classificationPathName,
                          int numClustersToExtract,
                          int percentToShave,
                          int numPermutations,
                          int gapTolerance,
                          boolean isWriteMatrix,
                          int numRows,
                          int numColumns,
                          double amplitude,
                          String model,
                          int noise,
                          int degreesFreedom)
    {
        super(dialog,
              isPerformSupervised,
              percentSupervision,
              classificationPathName,
              numClustersToExtract,
              percentToShave,
              numPermutations,
              gapTolerance,
              isWriteMatrix);

        this.numRows = numRows;
        this.numColumns = numColumns;
        this.amplitude = amplitude;
        this.model = model;
        this.noise = noise;
        this.degreesFreedom = degreesFreedom;

        setDemoFuncName();
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Writes all S interpreter simulation input to the output stream.
     *
     * @param bw  output stream writer
     */
    public void writeSinterpreterInput(SBufferedWriter bw)
    {
        try
        {
            synchronized(bw)
            {
		bw.TimeStamp ("GeneClust demo input");
                writeSinterpreterInit(bw);
		bw.Source ("GCDemos.s");
		bw.Source (demoFuncName + ".s");
                writeSinterpreterGenericInput(bw);
                writeSinterpreterDemoInput(bw);

                bw.flush();
            }
        }
        catch (IOException ioe)
        {
            System.err.println(ioe);
        }
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Sets the S interpreter function name to be invoked for this simulation.
     * The name should reflect the name of an existing S interpreter source file
     * (sans extension).
     */
    private void setDemoFuncName()
    {
        this.demoFuncName = "GC" + "Demo" + model;
    }

    /**
     * Writes mode-specific S interpreter input data to the output stream.
     *
     * @param bw  output stream writer
     * @throws IOException if an error occurs while writing
     */
    private void writeSinterpreterDemoInput(SBufferedWriter bw)
      throws IOException
    {
        bw.write("N<-"     + numRows);
        bw.newLine();
        bw.write("p<-"     + numColumns);
        bw.newLine();
        bw.write("a<-"     + amplitude);
        bw.newLine();
        bw.write("noise<-" + noise);
        bw.newLine();
        bw.write("df<-"    + degreesFreedom);
        bw.newLine();
        String sinterpreterFuncName = demoFuncName + ".func";
        bw.write("A<-" +
                 sinterpreterFuncName +
                 "(nclst,percentToShave,nperm,gapTolerance,N,p,a,");
        bw.newLine();
        bw.write(blankPadding(sinterpreterFuncName.length()+4) +
                 "noise,df,percentSupervision,classfile);");
        bw.newLine();
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private int numRows;
    private int numColumns;
    private double amplitude;
    private String model;
    private String demoFuncName;
    private int noise;
    private int degreesFreedom;
}


/**
 * RawDataSimulation - Data passing between the user interface and Sinterpreter when
 *                     running in raw data mode.
 *
 * @version 0.95 2001/02/20
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
final class RawDataSimulation extends Simulation
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor specifying (way too many) simulation parameters...
     *
     * @param dialog                  dialog allows monitoring of sim progress
     * @param isPerformSupervised     if true, perform supervised shaving
     * @param percentSupervision      percent supervision to use
     * @param classificationPathName  classification for supervised shaving
     * @param numClustersToExtract    number of clusters to extract
     * @param percentToShave          percent to shave
     * @param numPermutations         number of permutations
     * @param gapTolerance            gap tolerance
     * @param isWriteMatrix           if true, write matrix to disk. This
     *                                should be disabled here as no benefit
     *                                is achieved.
     * @param rawDataFileName         filename containing data to be examined
     * @throws NullPointerException if dialog is null
     */
    public RawDataSimulation(SimulationDialog dialog,
                             boolean isPerformSupervised,
                             int percentSupervision,
                             String classificationPathName,
                             int numClustersToExtract,
                             int percentToShave,
                             int numPermutations,
                             int gapTolerance,
                             boolean isWriteMatrix,
                             String rawDataFileName)
    {
        super(dialog,
              isPerformSupervised,
              percentSupervision,
              classificationPathName,
              numClustersToExtract,
              percentToShave,
              numPermutations,
              gapTolerance,
              isWriteMatrix);

        this.rawDataFileName = rawDataFileName;
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Writes all S interpreter simulation input to the output stream.
     *
     * @param bw  output stream writer
     */
    public void writeSinterpreterInput(SBufferedWriter bw)
    {
        try
        {
            synchronized(bw)
            {
		bw.TimeStamp ("GeneClust rawdata input");
                writeSinterpreterInit(bw);
		bw.Source ("GCRawData.s");
                writeSinterpreterGenericInput(bw);
                writeSinterpreterRawDataInput(bw);

                bw.flush();
            }
        }
        catch (IOException ioe)
        {
            System.err.println(ioe);
        }
    }


    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Writes mode-specific S interpreter input data to the output stream.
     *
     * @param bw  output stream writer
     * @throws IOException if an error occurs while writing
     */
    private void writeSinterpreterRawDataInput(SBufferedWriter bw)
      throws IOException
    {
	bw.SetStringVar ("fname", rawDataFileName);
        String sinterpreterFuncName = "GCRawData.func";
        bw.write("A<-" +
                 sinterpreterFuncName +
                 "(nclst,percentToShave,nperm,gapTolerance,fname,");
        bw.newLine();
        bw.write(blankPadding(sinterpreterFuncName.length()+4) +
                 "percentSupervision,classfile);");
        bw.newLine();
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private String rawDataFileName;
}

