/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (Sinterpreter)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)Sinterpreter.java    0.95 2001/01/29
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.MessageFormat;
import java.util.ResourceBundle;


/**
 * Sinterpreter - Data processing portion of the application.
 *
 * @version 0.95 2001/01/29
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class Sinterpreter
    implements SinterpreterKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying simulation
     *
     * @param simulation  reference to simulation object
     * @throws NullPointerException if simulation object is null
     */
    public Sinterpreter(SProcess simulation)
    {
        if (simulation == null)
            throw new NullPointerException("simulation");

        this.simulation = simulation;
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Launches the external S interpreter process, sets up process monitoring and
     * logging threads, then writes the simulation input to the output stream.
     *
     * @throws Exception if anything goes wrong
     */
    public void runSimulation()
      throws Exception
    {
        process = null;

        try
        {
            // Launch process
            String systemCmd;
	    
	    if (pseudoTerminalExec.startsWith("/"))
	    systemCmd = pseudoTerminalExec + " " + sinterpreterExec;
	    else
	    systemCmd = System.getProperty("gcinstall.home") + "/" + pseudoTerminalExec + " " + sinterpreterExec;

            process = runtime.exec(systemCmd);

            // Create thread to wait for process to die
            reaperThread = new Thread(new SinterpreterReaper(), "ReaperThread");
            reaperThread.start();

            // Determine where to write output
            String outputPathName;
            {
                final String keyGCOutputDir = "geneclust.output";
                File outputDir = new File(System.getProperty(keyGCOutputDir));
                final String outputFileName = "sinterpreter.out";
                File outputPath = new File(outputDir, outputFileName);

                outputPathName = outputPath.getAbsolutePath();
            }

            // Create thread to read process output
            InputStream is = process.getInputStream();
            FileOutputStream fos = new FileOutputStream(outputPathName);
            SimulationDialog simulationDialog = simulation.getDialog();
            GCStreamMonitor streamMonitor =
                               new GCStreamMonitor(is, fos, simulationDialog);
// :TODO: Set this up to be activated via cmd-line option
            streamMonitor.enableEcho();    // make class emulate 'tee'
            streamMonitorThread = new Thread(streamMonitor,
                                             "StreamMonitorThread");
            streamMonitorThread.start();

            // Create stream to interactively communicate with process
            OutputStream os = process.getOutputStream();
            OutputStreamWriter osw = new OutputStreamWriter(os);
            final int BUFFER_SIZE = 128;
            bw = new SBufferedWriter(osw, BUFFER_SIZE);
            writeSinterpreterInput(bw);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw e;
        }
    }

    /**
     * Causes the S interpreter input required to interrupt the simulation to be
     * written to the output stream.
     *
     * @see org.mdanderson.biostat.geneclust.SProcess#writeSinterpreterInterrupt
     */
    public void interrupt()
    {
        simulation.writeSinterpreterInterrupt(bw);
    }

    /**
     * Ends the simulation by invoking simulation termination, and performs
     * some thread management cleanup.
     *
     * @see org.mdanderson.biostat.geneclust.SProcess#writeSinterpreterTermination
     */
    public void endSimulation()
    {
        simulation.writeSinterpreterTermination(bw);
        if (reaperThread.isAlive())
        {
            try
            {
                String formatStr = getBundleString(WAITING_FOR_THREAD_JOIN);
                Object[] args = { reaperThread };
                MessageFormat mf = new MessageFormat(formatStr);
                String message = mf.format(args);

                System.err.println(message);
                reaperThread.join();
                System.err.println(getBundleString(THREAD_JOINED));
            }
            catch (InterruptedException ie)
            {
                System.err.println(ie);
            }
        }

        if (streamMonitorThread.isAlive())
        {
            try
            {
                String formatStr = getBundleString(WAITING_FOR_THREAD_JOIN);
                Object[] args = { streamMonitorThread };
                MessageFormat mf = new MessageFormat(formatStr);
                String message = mf.format(args);

                System.err.println(message);

                // Close lowest level input stream and interrupt thread
                try
                {
                    InputStream is = process.getInputStream();
                    is.close();
                }
                catch (IOException ioe)
                {
                    System.err.println(ioe);
                }

                streamMonitorThread.interrupt();
                streamMonitorThread.join();
                System.err.println(getBundleString(THREAD_JOINED));
            }
            catch (InterruptedException ie)
            {
                System.err.println(ie);
            }
        }

        reaperThread = null;
        streamMonitorThread = null;
    }

    // =========================================================================
    // PRIVATE METHODS
    // =========================================================================

    /**
     * Causes all S interpreter simulation input to be written to the output
     * stream.
     *
     * @param bw  output stream writer
     * @see SProcess#writeSinterpreterInput 
     */
    private void writeSinterpreterInput(SBufferedWriter bw)
    {
        simulation.writeSinterpreterInput(bw);
    }

    // =========================================================================
    // INNER CLASS
    // =========================================================================

    /**
     * SinterpreterReaper - Runnable to handle waiting for process completion.
     *
     * @version 0.95 2001/01/29
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class SinterpreterReaper
        implements Runnable
    {
        // =====================================================================
        // IMPLEMENT RUNNABLE
        // =====================================================================

        /**
         * Thread entry-point for SinterpreterReaper object.
         * Waits for the S interpreter external process to terminate.
         */
        public void run()
        {
            try
            {
                String formatStr = getBundleString(WAITING_FOR_PROCESS);
                Object[] args = { process };
                MessageFormat mf = new MessageFormat(formatStr);
                String message = mf.format(args);

                System.err.println(message);
                process.waitFor();
                System.err.println(getBundleString(PROCESS_COMPLETED));
            }
            catch (InterruptedException ie)
            {
                System.err.println(ie);
            }
            finally
            {
                process.destroy();
                System.err.println(getBundleString(PROCESS_DESTROYED));
            }
        }
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;
    private static String sinterpreterExec;
    private static String sinterpreterName;
    private static String pseudoTerminalExec;

    static
    {
        final String propertyFile = Sinterpreter.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);

        final String keySinterpreterName = "sinterpreter.name";
        sinterpreterName = System.getProperty(keySinterpreterName);

        final String keySinterpreterExec = "sinterpreter.exec";
        sinterpreterExec = System.getProperty(keySinterpreterExec);
        pseudoTerminalExec = getBundleString(EXEC_PSEUDO_TERMINAL);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private File sinterpreterInputFile;
    private SProcess simulation;
    private volatile Process process;
    private volatile Thread reaperThread;
    private volatile Thread streamMonitorThread;
    private SBufferedWriter bw;
    private final Runtime runtime = Runtime.getRuntime();
}
