/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for validated non whitespace textarea class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)ContentTextArea.java    1.0 2002/03/17
 *
 * Copyright (c) 2002 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import javax.swing.JTextArea;
import javax.swing.plaf.metal.MetalLookAndFeel;


/**
 * ContentTextArea - Allows non whitespace validation of a textfield.
 *
 * @version 1.0 2002/03/17
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class ContentTextArea extends JTextArea
  implements StatusListener
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor
     */
    public ContentTextArea()
    {
        super();
        initValidation();
    }

    /**
     * Class constructor specifying text
     *
     * @param text  string containing initial value
     */
    public ContentTextArea(String text)
    {
        super(text);
        initValidation();
    }

    /**
     * Class constructor specifying number of rows and columns
     *
     * @param rows     number of rows >= 0
     * @param columns  number of columns >= 0
     * @throws IllegalArgumentException  if the rows or columns arguments
     *                                   are negative
     */
    public ContentTextArea(int rows, int columns)
    {
        super(rows, columns);
        initValidation();
    }

    /**
     * Class constructor specifying text, and number of rows and columns
     *
     * @param text     string containing initial value
     * @param rows     number of rows >= 0
     * @param columns  number of columns >= 0
     * @throws IllegalArgumentException  if the rows or columns arguments
     *                                   are negative
     */
    public ContentTextArea(String text, int rows, int columns)
    {
        super(text, rows, columns);
        initValidation();
    }

    // =========================================================================
    // IMPLEMENTS STATUSLISTENER
    // =========================================================================

    /**
     * Notifies observers when the state of the data changes.
     * Swaps the foreground and background colors depending on data validity.
     *
     * @param event  status event
     */
    public void statusChanged(StatusEvent event)
    {
        if (event.getStatus())
        {
            setBackground(MetalLookAndFeel.getWhite());
            setForeground(MetalLookAndFeel.getBlack());
            setCaretColor(MetalLookAndFeel.getBlack());
        }
        else
        {
            setBackground(MetalLookAndFeel.getBlack());
            setForeground(MetalLookAndFeel.getWhite());
            setCaretColor(MetalLookAndFeel.getWhite());
        }
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Adds its non whitespace validator as a status listener for
     * notification of state of data changes
     *
     * @param listener  status listener
     * @see ValidationAdapter#addStatusListener
     */
    public void addStatusListener(StatusListener listener)
    {
        nwv.addStatusListener(listener);
    }

    // =========================================================================
    // PACKAGE INTERFACE
    // =========================================================================

    /**
     * Initializes the validation features for this textfield
     */
    void initValidation()
    {
        // Asynchronous validation
        nwv = new NonWhitespaceValidator();
        addStatusListener(this);

        getDocument().addDocumentListener(nwv);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    NonWhitespaceValidator nwv;
}

