/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for extended option pane class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)ExtendedOptionPane.java    1.0 2002/03/01
 *
 * Copyright (c) 2002 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Container;
import java.awt.Dimension;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;


/**
 * NoticeDialog - Creates a generic NOTICE dialog.
 *
 * @version 1.0 2002/03/01
 * @author <a href="mailto:roebuck@mdanderson.org>P. Roebuck</a>
 */
public class ExtendedOptionPane extends JOptionPane
{
    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    public NoticeDialog createNoticeDialog(Container parent,
                                           String title,
                                           String message,
                                           long delay)
    {
        return new NoticeDialog(parent, title, message, delay);
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    public static void showNoticeDialog(Container parent,
                                        String title,
                                        String message,
                                        long delay)
    {
        ExtendedOptionPane pane = new ExtendedOptionPane();
        NoticeDialog dialog = pane.createNoticeDialog(parent,
                                                      title,
                                                      message,
                                                      delay);
        dialog.display();
    }

    public static void showInternalNoticeDialog(Container parent,
                                                String title,
                                                String message,
                                                long delay)
    {
        showNoticeDialog(parent, title, message, delay);
    }

    // =========================================================================
    // INNER CLASSES
    // =========================================================================

    /**
     * NoticeDialog - Creates a generic NOTICE dialog.
     *
     * @version 1.0 2002/03/01
     * @author <a href="mailto:roebuck@mdanderson.org>P. Roebuck</a>
     */
    class NoticeDialog
    {
        // =====================================================================
        // CONSTRUCTORS
        // =====================================================================

        public NoticeDialog(Container parent,
                            String title,
                            String message,
                            long delay)
        {
            JOptionPane pane = new JOptionPane(message,
                                   JOptionPane.INFORMATION_MESSAGE);
            pane.setOptions(new Object[0]); // Sets buttons to none

            if (parent instanceof JDesktopPane)
            {
                JInternalFrame frame = pane.createInternalFrame(parent, title);
                frame.setDefaultCloseOperation(
                                          JInternalFrame.DO_NOTHING_ON_CLOSE);
                dialog = frame;
            }
            else
            {
                JDialog jdialog = pane.createDialog(parent, title);
                jdialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
                dialog = jdialog;
            }

            Runnable runnable = new NoticeRunnable(delay);
            new Thread(runnable, "NoticeThread").start();
        }

        // =====================================================================
        // PUBLIC INTERFACE
        // =====================================================================

        public void display()
        {
            dialog.setVisible(true);
        }

        // =====================================================================
        // INNER CLASSES
        // =====================================================================

        /**
         * NoticeRunnable - Runnable to handle notice dialog popdown
         */
        class NoticeRunnable implements Runnable
        {
            // =================================================================
            // CONSTRUCTORS
            // =================================================================

            /**
             * Class constructor specifying display time delay
             *
             * @param delay  milliseconds to display the dialog
             */
            public NoticeRunnable(long delay)
            {
                this.delay = delay;
            }

            // =================================================================
            // IMPLEMENT RUNNABLE
            // =================================================================

            public void run()
            {
                try
                {
                    Thread.sleep(delay);
                }
                catch (InterruptedException ie)
                {
                    // Don't care
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        dialog.setVisible(false);
                    }
                });
            }

            // =================================================================
            // OBJECT ATTRIBUTES
            // =================================================================

            private long delay;
        }

        // =====================================================================
        // OBJECT ATTRIBUTES
        // =====================================================================

        protected Container dialog;
        protected Runnable runnable;
    }

    // =========================================================================
    // MAIN
    // =========================================================================

    /**
     * Main entry-point for the ExtendedOptionPane class
     *
     * @param argv  string array containing command line arguments
     */
    public static void main(String[] argv)
    {
        JFrame frame = new JFrame();
        JDesktopPane desktop = new JDesktopPane();
        frame.getContentPane().add(desktop);
        frame.setSize(new Dimension(300, 300));
        frame.setVisible(true);
        
        showNoticeDialog(frame,
                         "Heads Up!",
                         "New email has arrived...",
                         THREE_SECONDS);
        showInternalNoticeDialog(desktop,
                                 "Heads Up!",
                                 "New email has arrived...",
                                 THREE_SECONDS);

        // Wait long enough we see the second dialog
        try
        {
            Thread.sleep(THREE_SECONDS);
        }
        catch (InterruptedException ie)
        {
        }
        System.exit(0);
    }

    // =========================================================================
    // CONSTANTS
    // =========================================================================

    public static final long ONE_SECOND    = 1000L;
    public static final long TWO_SECONDS   = 2 * ONE_SECOND;
    public static final long THREE_SECONDS = 3 * ONE_SECOND;
    public static final long FOUR_SECONDS  = 4 * ONE_SECOND;
    public static final long FIVE_SECONDS  = 5 * ONE_SECOND;

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected NoticeDialog dialog;
}

