/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for abstract textfield validation adapter
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)ValidationAdapter.java    1.0 2000/12/27
 *
 * Copyright (c) 2000 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.util.Vector;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;


/**
 * ValidationAdapter - Textfield validation adapter
 *
 * @version 1.0 2000/12/27
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public abstract class ValidationAdapter
  implements Validator, StatusSource
{
    // =========================================================================
    // IMPLEMENTS VALIDATOR
    // =========================================================================

    /**
     * Determines the validity of data
     *
     * @param value  data to be checked
     * @return       true if data is valid; otherwise, false
     */
    public abstract boolean isValid(Object value);

    /**
     * Determines the validity of data and fires an event if state has changed
     *
     * @param value  data to be checked
     * @return       true if data is valid; otherwise, false
     * @see #isValid
     */
    public boolean validate(Object value)
    {
        boolean isValidNow = isValid(value);

        // Has state changed?
        if (isValidNow != wasValid)
        {
            fireStatusChanged(value, isValidNow);
            wasValid = isValidNow;
        }

        return isValidNow;
    }

    // =========================================================================
    // IMPLEMENTS DOCUMENTLISTENER
    // =========================================================================

    /**
     * Gives notification that an attribute or set of attributes changed
     *
     * @param event  document change notification event
     */
    public void changedUpdate(DocumentEvent event)
    {
        validateDocument(event.getDocument());
    }

    /**
     * Gives notification that there was an insert into the document
     *
     * @param event  document change notification event
     */
    public void insertUpdate(DocumentEvent event)
    {
        validateDocument(event.getDocument());
    }

    /**
     * Gives notification that a portion of the document has been removed
     *
     * @param event  document change notification event
     */
    public void removeUpdate(DocumentEvent event)
    {
        validateDocument(event.getDocument());
    }

    // =========================================================================
    // IMPLEMENTS STATUSSOURCE
    // =========================================================================

    /**
     * Adds a status listener for notification of state of data changes
     * 
     * @param listener  status listener
     */
    public void addStatusListener(StatusListener listener)
    {
        listeners.add(listener);
    }

    /**
     * Removes a status listener
     * 
     * @param listener  status listener
     */
    public void removeStatusListener(StatusListener listener)
    {
        listeners.remove(listener);
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Initiates the validation process on this component's text
     *
     * @param document  container for text (Swing text component model)
     */
    private void validateDocument(Document document)
    {
        try
        {
            validate(document.getText(0, document.getLength()));
        }
        catch (BadLocationException e)
        {
            // Don't care
        }
    }

    /**
     * Notifies all listeners that have registered interest for notification
     * on state of data changes. The listener list is processed last to first.
     *
     * @param value   value to be checked for validity
     * @param status  true if the data is valid; otherwise, false
     */
    private void fireStatusChanged(Object value, boolean status)
    {
        StatusEvent event = new StatusEvent(this, value, status);

        for (int i = listeners.size()-1; i >= 0; i--)
        {
            StatusListener sl = (StatusListener) listeners.elementAt(i);

            sl.statusChanged(event);
        }
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    Vector listeners = new Vector();
    boolean wasValid = false;
}

