/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for vertical flow layout manager
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)VerticalFlowLayout.java    1.0 2000/01/10
 *
 * Copyright (c) 2000 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;


/**
 * VerticalFlowLayout - Vertical flow layout manager
 *
 * @version 1.0 2000/12/27
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class VerticalFlowLayout
    implements LayoutManager
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor
     */
    public VerticalFlowLayout()
    {
        this(DEFAULT_VERTICAL_GAP);
    }

    /**
     * Class constructor specifying vertical gap
     *
     * @param verticalGap  gap to leave between vertically stacked components 
     */
    public VerticalFlowLayout(int verticalGap)
    {
        this.verticalGap = verticalGap;
    }

    // =========================================================================
    // IMPLEMENTS LAYOUTMANAGER
    // =========================================================================

    /**
     * If the layout manager uses a per-component string, adds the component
     * to the layout, associating it with the string argument. Unused.
     *
     * @param name       string to be associated with the component
     * @param component  component to be added
     */
    public void addLayoutComponent(String name, Component component)
    {
        // Unnecessary since we are not using the name to
        // differentiate components like some layout managers do.
    }

    /**
     * Removes the specified component from the layout.
     *
     * @param component  component to be added
     */
    public void removeLayoutComponent(Component component)
    {
    }

    /**
     * Calculates the preferred size dimensions for the specified container,
     * given the components it contains.
     *
     * @param parent  container to be laid out
     * @return        dimension containing preferred size of the container
     */
    public Dimension preferredLayoutSize(Container parent)
    {
        return getLayoutSize(parent, false);
    }

    /**
     * Calculates the minimum size dimensions for the specified container,
     * given the components it contains.
     *
     * @param parent  container to be laid out
     * @return        dimension containing minimum size of the container
     */
    public Dimension minimumLayoutSize(Container parent)
    {
        return getLayoutSize(parent, true);
    }

    /**
     * Lays out the specified container
     *
     * @param parent  container to be laid out
     */
    public void layoutContainer(Container parent)
    {
        Dimension available = parent.getSize();
        Dimension required = preferredLayoutSize(parent);
        boolean useMinimum = required.height < available.height;
        Insets insets = parent.getInsets();
        final int x = insets.left;
        int y = insets.top;
        final int w = Math.max(available.width, required.width);
        final int excessHeight = available.height - required.height;
        int count = parent.getComponentCount();

        for (int i = 0; i < count; i++)
        {
            Component component = parent.getComponent(i);

            if (component.isVisible())
            {
                int h = (useMinimum) ? component.getMinimumSize().height :
                                       component.getPreferredSize().height;

                if (excessHeight > 0)
                    h += (h * excessHeight / required.height);
                component.setBounds(x, y, w, h);
                y += (h + verticalGap);
            }
        }
    }

    // =========================================================================
    // PRIVATE METHODS
    // =========================================================================

    /**
     * Calculate the size that the container will require using this
     * layout manager.
     *
     * @param parent      container to be laid out
     * @param useMinimum  if true, calculate minimum size;
     *                    otherwise, calculate preferred size
     * @return            dimension representing required container size
     */
    private Dimension getLayoutSize(Container parent, boolean useMinimum)
    {
        int iNumComponents = parent.getComponentCount();
        Dimension outDim = new Dimension(0, 0);

        for (int i = 0; i < iNumComponents; i++)
        {
            Component c = (Component) parent.getComponent(i);
            Dimension cDim = (useMinimum) ? c.getMinimumSize() :
                                            c.getPreferredSize();

            // width will be width of largest component
            outDim.width = Math.max(cDim.width, outDim.width);
            outDim.height += cDim.height;
            if (i != 0)
                outDim.height += verticalGap;
        }

        Insets border = parent.getInsets();
        outDim.width += border.left + border.right;
        outDim.height += border.right + border.bottom;

        return outDim;
    }

    // =========================================================================
    // CONSTANTS
    // =========================================================================

    private static final int DEFAULT_VERTICAL_GAP = 2;

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    int verticalGap;
}

